#!/usr/bin/python3
# -*- coding: utf-8 -*-
import gc
import logging
import dbus
import dbus.service
import dbus.mainloop.glib
import os
import time
import datetime
import threading
import shutil
import apt_pkg, apt
from apt.debfile import DebPackage
from kylindbus.apt_handler import AppActions, AptHandler, WorkitemError
from utils.log_manager import LogManager
from utils.enums import INTERFACE, INTERFACES, AUTOSTART, CRONDAILY
from kylindbus.policykit_service import PolicyKitService
import configparser
import aptsources.sourceslist
import subprocess
from PyQt5.QtCore import pyqtSignal, QSettings
from xdg import BaseDirectory as xdg

CRONPATH = "/var/spool/anacron/"
SOURCESPATH = "/etc/apt/sources.list.d"
CODEHOME = "/usr/share/kylin-update-manager/"
DOWN_CACHE_PATH = os.path.join(xdg.xdg_cache_home, "kylin-update-manager/kylin-update-manager-deb")
DOWN_TEMP_PATH = os.path.join(xdg.xdg_cache_home, "kylin-update-manager/kylin-update-manager-deb/deb")

class WorkItem:
     def __init__(self, pkgname, action, kwargs):
        self.pkgname = pkgname
        self.action = action
        self.kwargs = kwargs

class WorkThread(threading.Thread):

    def __init__(self, dbusDaemon):
        threading.Thread.__init__(self)
        self.dbusDaemon = dbusDaemon
        ("KUM DBUS START ", type(self.dbusDaemon))

    #
    # 函数:线程执行函数
    #
    def run(self):
        while(True):

            if len(self.dbusDaemon.worklist) == 0:
                time.sleep(0.2)
                continue

            self.dbusDaemon.mutex.acquire()
            item = self.dbusDaemon.worklist.pop(0)
            self.dbusDaemon.mutex.release()
            try:
                func = getattr(self.dbusDaemon.aptHandler, item.action)
                if func is None:
                    print("Error action: ", item)
                res = func(item.pkgname,item.kwargs)
                if res is False:
                    print("Action exec failed...")
            except WorkitemError as e:
                # 如果是包列表，此时强制装换为str，后面显示str将出现异常，故而暂不显示包列表
                kwarg = {"apt_appname": str(item.pkgname),
                        "apt_percent": str(-e.errornum),
                        "action": str(item.action),
                        }
                self.dbusDaemon.kum_apt_signal("apt_error", kwarg)
            except:
                kwarg = {"apt_appname": str(item.pkgname),
                        "apt_percent": '-24',
                        "action": str(item.action),
                        }
                self.dbusDaemon.kum_apt_signal("apt_error", kwarg)
            time.sleep(0.3)


class Daemon(PolicyKitService):
    down_load_pid = ""
    def __init__ (self, bus, mainloop):
        #signal.signal(signal.SIGCHLD,signal.SIG_IGN)
        self.aptHandler = AptHandler(self)
        self.worklist = []
        self.cancel_name_list = []
        self.mutex = threading.RLock()
        self.cancelmutex = threading.RLock()
        self.worker_thread = WorkThread(self)
        self.worker_thread.setDaemon(True)
        self.worker_thread.start()
        self.efi_status = ""


        self.bus_name = dbus.service.BusName(INTERFACE, bus=bus)
        PolicyKitService.__init__(self, self.bus_name, INTERFACES)
        self.mainloop = mainloop

    #
    # 函数:停止dbus
    #
    def stop(self):
        """ stop the dbus controller and remove from the bus """
        self.remove_from_connection()

    #
    # 函数:新增任务
    # item：任务
    #
    def add_worker_item(self, item):
        print("####add_worker_item:", item)
        self.mutex.acquire()
        self.worklist.append(item)
        self.mutex.release()

    #
    # 函数:删除任务
    # pkgname：任务名
    #
    def del_worker_item_by_name(self, pkgname):
        print("####del_worker_item_by_name:", pkgname)
        exist = False
        self.mutex.acquire()
        for item in self.worklist:
            if item.pkgname == pkgname:
                exist = True
                break
        if exist is True:
            self.worklist.remove(pkgname)
        self.mutex.release()

        self.cancelmutex.acquire()
        self.cancel_name_list.append(pkgname)
        self.cancelmutex.release()
        print("####del_worker_item_by_name finished!")

    #
    # 函数:检查取消的任务
    #
    def check_cancel_worker_item(self, pkgname):
#        print "####check_cancel_worker_item:",pkgname
        cancel = False
        self.cancelmutex.acquire()
#        print "check_cancel_worker_item:",len(self.cancel_name_list)
        for item in self.cancel_name_list:
            if item == pkgname:
                cancel = True
                break
        if cancel is True:
            self.cancel_name_list.remove(pkgname)
        self.cancelmutex.release()

#        print "####check_cancel_worker_item finished!:",cancel
        return cancel
    #
    # 函数：共用函数用于设置apt状态并发送信号。
    #
    def send_apt_status(self, name, percent, message, status=True):
        if status:
            kwarg = {"apt_appname": str(name), "apt_percent": str(
                percent), "action": str(message)}
            self.kum_apt_signal("apt_finish", kwarg)
        else:
            kwarg = {"apt_appname": str(name), "apt_percent": str(
                percent), "error_message": str(message)}
            self.kum_apt_signal("apt_error", kwarg)
        self.set_efi_status(False)
    #
    # 函数：判断efi分区的状态
    #
    def get_efi_status(self):
        with open("/proc/mounts") as f:
            for line in f.readlines():
                line_section = line.split(" ")
                if "/boot/efi" in line_section[1]:
                    status = line_section[3].split(",")
                    for i in status:
                        if i == "ro" or i == "rw":
                            return i
                    break

    #
    # 函数：安装前后设置efi分区的状态为读写
    #
    def set_efi_status(self, before=True):
        try:
            if(self.efi_status == "ro" and before == True):
                os.system("mount -o remount,rw /boot/efi")
                os.system("mount -o remount,rw /boot")
            elif(self.efi_status == "ro" and before == False):
                os.system("mount -o remount,ro /boot/efi")
                os.system("mount -o remount,ro /boot")
        except Exception as e:
            LogManager.ERR("修改/boot/efi挂载状态出错")
            #logger.error("修改/boot/efi挂载状态出错", exc_info=True)
    #
    # 函数:退出dbus
    #
    @dbus.service.method(INTERFACE, in_signature='', out_signature='')
    def exit(self):
        self.mainloop.quit()

    #
    # 函数:初始化cache
    #
    @dbus.service.method(INTERFACE, in_signature='', out_signature='')
    def init_cache(self):
        self.aptHandler.init_cache()

    #
    # 函数:安装和升级
    # install package sa:kum_fetch_signal() and kum_apt_signal()
    #
    @dbus.service.method(INTERFACE, in_signature='s', out_signature='b', sender_keyword='sender')
    def install_and_upgrade(self, pkgName, sender=None):
        print("####install_and_upgrade: ", pkgName)
        
        item = WorkItem(pkgName, AppActions.INS_AND_UP, None)

        self.add_worker_item(item)

        print("####install_and_upgrade return")
        return True

    #
    # 函数：安装
    # install package sa:kum_fetch_signal() and kum_apt_signal()
    #
    @dbus.service.method(INTERFACE, in_signature='as', out_signature='b', sender_keyword='sender')
    def install(self, pkgNames, sender=None):
        print("####install: ", pkgNames)

        item = WorkItem(pkgNames, AppActions.INSTALL, None)

        self.add_worker_item(item)

        print("####install return")
        return True

    #
    # 函数：单个安装
    #
    @dbus.service.method(INTERFACE, in_signature='s', out_signature='b', sender_keyword='sender')
    def insone(self, pkgName, sender=None):
        print("####insone: ", pkgName)

        item = WorkItem(pkgName, AppActions.INSTALL_ONE, None)

        self.add_worker_item(item)

        print("####insone return")
        return True

    #
    # 函数：升级
    # update package sa:kum_fetch_signal() and kum_apt_signal()
    #
    @dbus.service.method(INTERFACE, in_signature='as', out_signature='b', sender_keyword='sender')
    def upgrade(self, pkgNames, sender=None):
        print("####upgrade: ", pkgNames)

        item = WorkItem(pkgNames, AppActions.UPGRADE, None)

        self.add_worker_item(item)

        print("####upgrade return")
        return True

    #
    # 函数：单个升级
    #
    @dbus.service.method(INTERFACE, in_signature='s', out_signature='b', sender_keyword='sender')
    def upone(self, pkgName, sender=None):
        print("####upone: ", pkgName)

        item = WorkItem(pkgName, AppActions.UPGRADE_ONE, None)

        self.add_worker_item(item)

        print("####upone return")
        return True

    #
    # 函数：取消
    #
    @dbus.service.method(INTERFACE, in_signature='s', out_signature='b', sender_keyword='sender')
    def cancel(self, pkgName, sender=None):
        print("####cancel: ", pkgName)

        self.del_worker_item_by_name(pkgName)

        print("####cancel return")
        return True

    #
    # 函数：更新软件源
    # apt-get update sa:kum_fetch_signal()
    #
    @dbus.service.method(INTERFACE, in_signature='b', out_signature='b', sender_keyword='sender')
    def update(self, quiet, sender=None):
        print("####update: ")

        kwargs = {"quiet":str(quiet),
                  }

        item = WorkItem("#update", AppActions.UPDATE, kwargs)

        self.add_worker_item(item)

        print("####update return")
        return True

    #
    # 函数：通过shell更新软件源
    #
    @dbus.service.method(INTERFACE, in_signature='b', out_signature='b', sender_keyword='sender')
    def update_by_shell(self, quiet, sender=None):
        print("####update by shell: ")

        kwargs = {"quiet":str(quiet),
                  }

        item = WorkItem("#update-shell", AppActions.UPDATESHELL, kwargs)

        self.add_worker_item(item)

        print("####update by shell return")
        return True

    #
    # 函数：解决冲突
    #
    @dbus.service.method(INTERFACE, in_signature='b', out_signature='b', sender_keyword='sender')
    def configure_dpkg_by_shell(self, quiet, sender=None):
        print("####configure dpkg by shell: ")

        kwargs = {"quiet":str(quiet),
                  }

        item = WorkItem("#configure-shell", AppActions.CONFIGURESHELL, kwargs)

        self.add_worker_item(item)

        print("####configure dpkg by shell return")
        return True

    #
    # 函数：通过shell解决冲突
    #
    @dbus.service.method(INTERFACE, in_signature='b', out_signature='b', sender_keyword='sender')
    def fix_conffile_by_shell(self, quiet, sender=None):
        print("####fix conffile by shell: ")

        kwargs = {"quiet":str(quiet),
                  }

        item = WorkItem("#fix-conffile-shell", AppActions.FIXCONFFILE, kwargs)

        self.add_worker_item(item)

        print("####fix conffile by shell return")
        return True

    #
    # 函数：开机自启动开启
    #
    @dbus.service.method(INTERFACE, in_signature='', out_signature='b', sender_keyword='sender')
    def autostart_open(self, sender=None):
        try:
            cmd = "cp " + CODEHOME + "kylin-update-manager-autostart.desktop  " + AUTOSTART
            os.popen(cmd)
            return True
        except:
            return False

    #
    # 函数：修改配置文件
    #
    @dbus.service.method(INTERFACE, in_signature='ssss', out_signature='', sender_keyword='sender')
    def modify_conf(self, path, group, key, value, sender=None):
        try:
            cf = configparser.ConfigParser()
            cf.read(path)
            cf.set(group, key, value)
            with open(path, "w+") as f:
                cf.write(f)
        except:
            return False

    #
    # 函数：开机自启动关闭
    #
    @dbus.service.method(INTERFACE, in_signature='', out_signature='b', sender_keyword='sender')
    def autostart_close(self, sender=None):
        try:
            cmd = "rm " + AUTOSTART + "kylin-update-manager-autostart.desktop"
            os.popen(cmd)
            return True
        except:
            return False

    #
    # 函数：每日定时启动开启
    #
    @dbus.service.method(INTERFACE, in_signature='', out_signature='b', sender_keyword='sender')
    def dailystart_open(self, sender=None):
        try:
            cmd = "cp " + CODEHOME + "0kylin-update-manager-daily  " + CRONDAILY
            os.popen(cmd)
            return True
        except:
            return False

    #
    # 函数：每日定时启动关闭
    #
    @dbus.service.method(INTERFACE, in_signature='', out_signature='b', sender_keyword='sender')
    def dailystart_close(self, sender=None):
        try:
            cmd = "rm " + CRONDAILY + "0kylin-update-manager-daily"
            os.popen(cmd)
            return True
        except:
            return False

    #-------------------------------------signals----------------------------------------

    @dbus.service.signal(INTERFACE, signature='sa{ss}')
    def software_auth_signal(self, type, msg):
        pass

    #
    # 函数：下载消息信号
    # package download status signal
    '''parm mean
        type:
            start:start download
            stop:all work is finish
            done:all items download finished
            fail:download failed
            fetch:one item download finished
            pulse:download status, this msg given a string like dict
        msg:
            a message of type, sometimes is None
    '''
    @dbus.service.signal(INTERFACE, signature='sa{ss}')
    def kum_fetch_signal(self, type_msg, msg):
        pass

    #
    # 函数：执行消息信号
    # package install/update/remove signal
    '''parm mean
        type:
            start:start work
            stop:work finish
            error:got a error
            pulse:work status, this msg given a string like dict
        msg:
            a message of type, sometimes is None
    '''
    @dbus.service.signal(INTERFACE, signature='sa{sv}')
    def kum_apt_signal(self, type_msg, msg):
        pass

    # test func
    @dbus.service.method(INTERFACE, in_signature='i', out_signature='i')
    def test_return_value(self, arg):
        return 2

    #
    # 函数：恢复默认sourceslist
    #
    @dbus.service.method(INTERFACE, in_signature='', out_signature='b')
    def change_source_list_to_default(self):

        if(os.path.isfile("/etc/apt/sources.list.kumbak")):
            shutil.move("/etc/apt/sources.list.kumbak", "/etc/apt/sources.list")

        # lixiang
        tmppath = '/var/lib/apt/tmp'
        aptpath = '/var/lib/apt/lists'

        if os.path.exists(tmppath) and os.path.isdir(tmppath):
            # remove /var/lib/apt/lists
            for filename in os.listdir(aptpath):
                if filename != 'lock':
                    abspath = os.path.join(aptpath, filename)
                    if os.path.isfile(abspath):
                        os.remove(abspath)

            # copy to /var/lib/apt/lists
            for filename in os.listdir(tmppath):
                if filename != 'lock':
                    abspath = os.path.join(tmppath, filename)
                    if os.path.isfile(abspath):
                        shutil.move(abspath, os.path.join(aptpath, filename))

            # delete /var/lib/apt/tmp
            shutil.rmtree(tmppath)

        return True

    #
    # 函数：恢复升级服务器的sourceslist
    #
    @dbus.service.method(INTERFACE, in_signature='as', out_signature='b')
    def change_source_list_to_kylin_update_server(self, lines):
        # print("chenggong")
        if os.path.exists(SOURCESPATH):
           # print("失败")
           for item in os.listdir(SOURCESPATH):
               targetFile = os.path.join(SOURCESPATH, item)
               if os.path.isfile(targetFile):
                   os.remove(targetFile)

        # wenbo 判断备份文件如果存在，说明上次可能异常中断没进行还原，这次不再进行源备份，防止覆盖上次真正备份的内容
        if(os.path.isfile("/etc/apt/sources.list.kumbak")==False):
            shutil.copy("/etc/apt/sources.list", "/etc/apt/sources.list.kumbak")

            # lixiang
            tmppath = '/var/lib/apt/tmp'
            aptpath = '/var/lib/apt/lists'

            # mkdir /var/lib/apt/tmp
            if os.path.exists(tmppath) and os.path.isdir(tmppath):
                shutil.rmtree(tmppath)
            os.mkdir(tmppath, 0o755)

            for filename in os.listdir(aptpath):
                if filename != 'lock':
                    abspath = os.path.join(aptpath, filename)
                    if os.path.isfile(abspath):
                        cmd = "cp %s %s" %(abspath ,os.path.join(tmppath, filename))
                        #shutil.copy(abspath, os.path.join(tmppath, filename))
                        os.system(cmd)
        fp = open('/etc/apt/sources.list', 'w+')
        fp.tell()
        for line in lines:
           fp.write(line + "\n")
        fp.close()

        return True

    #
    # 函数：wget下载软件包
    #
    @dbus.service.method(INTERFACE, in_signature='ass', out_signature='')
    def copy_file_to_install(self,src_list,appname):
        for i in src_list:
            if os.path.exists(os.path.join("/var/cache/apt/archives", os.path.basename(i))):
                pass
            else:
                shutil.move(i, os.path.join("/var/cache/apt/archives", os.path.basename(i)))
        self.copy_finish(appname)

    #
    #函数:判断拷贝结束
    #
    @dbus.service.signal(INTERFACE,signature="s")
    def copy_finish(self,appname):
        pass

    #
    #函数：暂停下载
    #
    @dbus.service.method(INTERFACE, in_signature='', out_signature='')
    def kill_pid(self):
        os.system("killall -9 wget")

    #
    #函数：点击取消下载
    #
    @dbus.service.method(INTERFACE, in_signature='s', out_signature='')
    def cancel_download_app(self,appname):
        cmd1 = "rm /var/cache/apt/archives/"+appname+"_*"
        os.system(cmd1)

    #
    #函数：获取依赖包
    #
    @dbus.service.method(INTERFACE, in_signature='s', out_signature='as')
    def get_depends_pkgs(self,appname):
        res = os.popen("sudo LANG= apt-get install --print-uris %s -y" %appname).readlines()
        return res

    #
    #函数：下载依赖包
    #
    @dbus.service.method(INTERFACE, in_signature='s', out_signature='')
    def down_dep_pkg(self,appname):
        cmd = "apt-get --download-only install "+appname+" -y"
        os.system(cmd)
        self.down_dep_finish(appname)

    #
    #信号：确认依赖包下载结束
    #
    @dbus.service.signal(INTERFACE, signature="s")
    def down_dep_finish(self,appname):
        pass

    #
    # 函数：安装软件包
    #
    @dbus.service.method(INTERFACE, in_signature='s', out_signature='b', sender_keyword='sender')
    def install_debfile(self, path, sender=None):
        # print("####install deb file: ", path)
        print("开始安装----- %s -----", path)
        self.efi_status = self.get_efi_status()
        self.set_efi_status(True)
        self.aptHandler.init_cache()
        # print("send auth signal")
        # 阶段一：检查路径是否存在
        path = str(path).strip().replace("\n", "")  # 防止路径传入\n和空格
        try:
            debfile = DebPackage(path, self.aptHandler.cache)
        except Exception as e:
            self.send_apt_status(path, 90, str(e), False)
            print(str(e))
            print("安装状态：失败 - 路径：" + path + " - 错误码：(90) - 详情：该路径不存在")
            del path
            gc.collect()
            return False
        # 阶段二：调用apt模块解析依赖并获取依赖
        kum = {"appname": debfile._sections["Package"],
               "version": debfile._sections["Version"],
               "description": debfile._sections["Description"],
               "icon": "",
               "keyword": str(2)}
        debfile.check()  # do debfile.check for the next to do debfile.missing_deps
        # print("len:%s" % len(debfile.missing_deps))
        # 安装源中存在的依赖包
        if len(debfile.missing_deps) != 0:
            print("开始解析依赖，需要的依赖为", debfile.missing_deps)
            self.aptHandler.init_cache()
            for deb in debfile.missing_deps:
                try:
                    pkg = self.aptHandler.get_pkg_by_name(deb)
                except Exception as e:
                    self.send_apt_status(deb, 99, e, False)
                    msg = {"statue": "Fail", "errorcode": str(99), "error_message": "安装本地包无法获取依赖%s" % deb}  # 记录结果写入数据库
                    print("安装状态：失败 - 路径：" + path + " - 错误码：(99) - 详情：安装本地包无法获取依赖%s" % deb)
                    del path, debfile
                    gc.collect()
                    return False
                pkg.mark_install()
                try:
                    self.aptHandler.cache.commit()
                except apt.cache.LockFailedException as e:
                    msg = {"statue": "Fail", "errorcode": str(98), "error_message": "dpkg被锁导致安装依赖包失败"}  # 记录结果写入数据库
                    self.send_apt_status(deb, 98, e, False)
                    print("安装状态：失败 - 路径：" + path + " - 错误码：(98) - 详情：dpkg被锁导致安装依赖包失败")
                    del path, debfile
                    gc.collect()
                    return False
                except SystemError as e:
                    msg = {"statue": "Fail", "errorcode": str(90), "error_message": str(e)}  # 记录结果写入数据库
                    self.send_apt_status(deb, 93, e, False)
                    print("安装状态：失败 - 路径：" + path + " - 错误码：(90) - 详情：%s" % str(e))
                    del path, debfile
                    gc.collect()
                    return False
        else:
            print("不需要依赖")

        # 阶段三：安装软件包
        # res = debfile.install(AptProcess(self, pkgName, AppActions.INSTALL_ONE,1,pkgName))
        try:
            print("开始安装软件包：", debfile._sections["Package"])
            res = debfile.install()
            if res:  # 安装失败
                print("安装软件包失败--->", debfile._sections["Package"])
                try:
                    self.aptHandler.init_cache()
                    pkg = self.aptHandler.get_pkg_by_name(kum["appname"])
                    cmd = "dpkg -l {} | cut -d ' ' -f 1  | tail -1".format(debfile._sections["Package"])
                    r = os.popen(cmd)
                    text = r.read()
                    r.close()
                    text = text.strip().replace("\n", "")
                    if pkg.is_installed and text != "ii":
                        pkg.mark_delete(True, True)  # 安装时失败后需要直接卸载防止apt出现依赖问题
                        self.aptHandler.cache.commit(None, None)
                except:
                    pass
                msg = {"statue": "Fail", "errorcode": str(97), "error_message": "deb包存在问题导致安装失败"}  # 记录结果写入数据库
                self.send_apt_status(kum["appname"], 97, "deb包存在问题导致安装失败", False)
                print("安装状态：失败 - 路径：" + path + " - 错误码：(97) - 详情：deb包存在问题导致安装失败")
                del path, debfile
                gc.collect()
                return False
            else:  # 安装成功
                msg = {"statue": "Success", "errorcode": str(0), "error_message": ""}  # 记录结果写入数据库
                self.send_apt_status(kum["appname"], 100, AppActions.INSTALL_DEBFILE)
                print("安装状态：成功 - 路径：" + path)
        except apt_pkg.Error as error:
            msg = {"statue": "Fail", "errorcode": str(96), "error_message": str(error)}  # 记录结果写入数据库
            self.send_apt_status(kum["appname"], 96, str(error), False)
            print("安装状态：失败 - 路径：" + path + " - 错误码：(96) - 详情：%s" % str(error))
            del path, debfile
            gc.collect()
            return False
        except SystemError as e:
            msg = {"statue": "Fail", "errorcode": str(80), "error_message": str(e)}  # 记录结果写入数据库
            self.send_apt_status(kum["appname"], 80, str(e), False)
            print("安装状态：失败 - 路径：" + path + " - 错误码：(80) - 详情：%s" % str(e))
            del path, debfile
            gc.collect()
            return False
        except Exception as e:
            msg = {"statue": "Fail", "errorcode": str(79), "error_message": str(e)}  # 记录结果写入数据库
            self.send_apt_status(kum["appname"], 79, str(e), False)
            print("安装状态：失败 - 路径：" + path + " - 错误码：(79) - 详情：%s" % str(e))
            del path, debfile
            gc.collect()
            return False
        except BaseException as e:
            msg = {"statue": "Fail", "errorcode": str(78), "error_message": str(e)}  # 记录结果写入数据库
            self.send_apt_status(kum["appname"], 78, str(e), False)
            print("安装状态：失败 - 路径：" + path + " - 错误码：(78) - 详情：%s" % str(e))
            del path, debfile
            gc.collect()
            return False
        except:
            print("安装的execpt函数")
        print("安装结束----- %s -----" % path)
        del path, debfile
        gc.collect()
        return True

    #
    # 函数：修改局域网域名地址
    #
    @dbus.service.method(INTERFACE, in_signature='s', out_signature='b')
    def change_intranet_domain_name(self, ip):
        try:
            settings = QSettings("/etc/kum.conf", QSettings.IniFormat)
            settings.setIniCodec("utf-8")
            settings.beginGroup("server_url")
            settings.setValue("intranet", str(ip))
            settings.endGroup()
            return True
        except:
            return False

    @dbus.service.method(INTERFACE, in_signature='s', out_signature='b', sender_keyword='sender')
    def remove(self, pkgName, sender=None):
        # print("####remove: ", pkgName)
        # 阶段一：初始化模块，获取软件信息
        self.efi_status = self.get_efi_status()
        self.set_efi_status(True)
        self.aptHandler.init_cache()
        pkgName = str(pkgName).strip().replace("\n","")
        try:
            pkg = self.aptHandler.get_pkg_by_name(pkgName)
        except Exception as e:
            self.send_apt_status(pkgName, 95, str(e), False)
            return False
        if pkg.is_installed is False:
            error_message = "%s未安装"%pkg
            self.send_apt_status(pkgName, 94, error_message, False)
            return False
        # 阶段二：开始卸载软件包
        pkg.mark_delete(True, True)
        try:
            self.aptHandler.cache.commit(None, None)
        except apt.cache.LockFailedException as e:
            self.send_apt_status(pkgName, 93, str(e), False)
            # raise WorkitemError(3, "package manager is running.")
        except Exception as e:
            os.system("dpkg --configure -a")
            self.send_apt_status(pkgName, 92, str(e), False)
        else:
            self.send_apt_status(pkgName,100,"remove_debfile")
        return True
